class Distribution:

    """ Generic distribution class for calculating and 
		visualizing a probability distribution.
	
		Attributes:
			mean (float) representing the mean value of the distribution
			stdev (float) representing the standard deviation of the distribution
			data_list (list of floats) a list of floats extracted from the data file
			"""
    def __init__(self, mu =0,  sigma=1):
        self.mean = mu
        self.std = sigma
        self.data = []

    def read_data_file(self, file_name, sample=True):

        """Function to read in data from a txt file. The txt file should have
		one number (float) per line. The numbers are stored in the data attribute.
				
		Args:
			file_name (string): name of a file to read from
		
		Returns:
			None
		
		"""

        with open(file_name) as f:
            datalist = []
            line = f.readline()
            while line:
                datalist.append(int(line))
                line = f.readline()
        f.close()

        self.data = datalist
        





import math
import matplotlib.pyplot as plt
from .Generaldistribution import Distribution

class Binomial(Distribution):

    """ Binomial distribution class for calculating and 
    visualizing a Binomial distribution.
    
    Attributes:
        mean (float) representing the mean value of the distribution
        stdev (float) representing the standard deviation of the distribution
        data_list (list of floats) a list of floats to be extracted from the data file
        p (float) representing the probability of an event occurring
        n (int) number of trials
            
    """

    def __init__(self, p: float = .5, n: int = 20):
        self.prob = p
        self.n_trial = n
        Distribution.__init__(self, self.calculate_mean(), self.calculate_stdev())
        

    def calculate_mean(self):

        """Function to calculate the mean from p and n
        
        Args: 
            None
        
        Returns: 
            float: mean of the data set
    
        """
        
        mu = round((self.prob * self.n_trial),2)
        self.mean = mu
        return mu


    def calculate_stdev(self):

        """Function to calculate the standard deviation from p and n.
        
        Args: 
            None
        
        Returns: 
            float: standard deviation of the data set
    
        """

        sigma = round((math.sqrt(self.n_trial * self.prob * (1 - self.prob))),2)
        self.std = sigma
        return sigma

    def __add__(self, other):

        """Function to add together two Binomial distributions with equal p
        
        Args:
            other (Binomial): Binomial instance
            
        Returns:
            Binomial: Binomial distribution
            
        """

        try:
            assert self.prob == other.prob
        except AssertionError as error:
            raise error

        result = Binomial()

        result.n_trial = self.n_trial + other.n_trial
        result.prob = self.prob
        result.calculate_mean()
        result.calculate_stdev()
        
        return result

    def __repr__(self):

        return f"mean {self.mean}, standard deviation {self.std}, p {self.prob}, n {self.n_trial}" 

    def replace_stats_with_data(self):

        """Function to calculate p and n from the data set
        
        Args: 
            None
        
        Returns: 
            float: the p value
            float: the n value
    
        """

        self.n_trial = len(self.data)
        self.prob = round((1.0 * sum(self.data) / len(self.data)),4)
        self.mean = self.calculate_mean()
        self.std = self.calculate_stdev()

        return self.prob, self.n_trial


    def plot_bar(self):

        """Function to output a histogram of the instance variable data using 
        matplotlib pyplot library.
        
        Args:
            None
            
        Returns:
            None
        """

        plt.bar(x = ['0', '1'], height =[(1 - self.prob) * self.n_trial, self.prob * self.n_trial])
        plt.title('Barplot of Data')
        plt.xlabel('outcome')
        plt.ylabel('count')
        plt.show()


    def pdf (self, k: float):

        """Probability density function calculator for the binomial distribution.
        
        Args:
            x (float): point for calculating the probability density function
            
        
        Returns:
            float: probability density function output
        """

        a = math.factorial(self.n_trial) / (math.factorial(k) * (math.factorial(self.n_trial - k)))
        b = (self.prob ** k) * (1 - self.prob) ** (self.n_trial - k)

        return a * b

    
    
    def plot_bar_pdf (self):

        """Function to plot the pdf of the binomial distribution
        
        Args:
            None
        
        Returns:
            list: x values for the pdf plot
            list: y values for the pdf plot
            
        """

        x = [ ]
        y = [ ]

        for i in range (self.n_trial + 1):
            x.append(i)
            y.append(self.pdf(i))


        plt.bar(x, y)
        plt.title('Distribution of Outcomes')
        plt.ylabel('Probability')
        plt.xlabel('Outcome')
        plt.show()


from .Generaldistribution import Distribution
import numpy as np
import matplotlib.pyplot as plt
import math

class Gaussian(Distribution):

    """ Gaussian distribution class for calculating and 
	visualizing a Gaussian distribution.
	
	Attributes:
		mean (float) representing the mean value of the distribution
		stdev (float) representing the standard deviation of the distribution
		data_list (list of floats) a list of floats extracted from the data file
			
	"""

    def __init__(self, mu=0, sigma =1):
        Distribution.__init__(self, mu, sigma)

    def calculate_mean(self):

        """Function to calculate the mean of the data set.
		
		Args: 
			None
		
		Returns: 
			float: mean of the data set
	
		"""

        mean = sum(self.data) / len(self.data)
        self.mean = mean
        return mean


    def calculate_stdev(self, sample = True):

        """Function to calculate the standard deviation of the data set.
		
		Args: 
			sample (bool): whether the data represents a sample or population
		
		Returns: 
			float: standard deviation of the data set
	
		"""

        arr = np.array(self.data)
        if sample:
            stdev = np.std(arr, ddof=1)
            self.std = stdev
            
        else:
            stdev = np.std(arr, ddof=0)
            self.std = stdev
        return stdev


    def __add__(self, other):

        """Function to add together two Gaussian distributions
		
		Args:
			other (Gaussian): Gaussian instance
			
		Returns:
			Gaussian: Gaussian distribution
			
		"""

        result = Gaussian()
        result.mean = self.mean + other.mean 
        result.std = math.sqrt(self.std ** 2 + other.std ** 2)
        return result


    def __repr__(self):

        """Function to output the characteristics of the Gaussian instance
		
		Args:
			None
		
		Returns:
			string: characteristics of the Gaussian
		
		"""

        return f"mean: {self.mean}, standard deviation: {self.std}"


    def plot_histogram(self):

        """Function to output a histogram of the instance variable data using 
		matplotlib pyplot library.
		
		Args:
			None
			
		Returns:
			None
		"""

        plt.hist(self.data)
        plt.title('Histogram of Data')
        plt.xlabel('data')
        plt.ylabel('count')
        plt.show()
    

    def pdf(self, x: float):

        """Probability density function calculator for the gaussian distribution.
		
		Args:
			x (float): point for calculating the probability density function
			
		
		Returns:
			float: probability density function output
		"""

        return (1.0 / (self.std * math.sqrt(2*math.pi))) * math.exp(-0.5*((x - self.mean) / self.std) ** 2)


    def plot_histogram_pdf(self, n_spaces: int):

        """Function to plot the normalized histogram of the data and a plot of the 
		probability density function along the same range
		
		Args:
			n_spaces (int): number of data points 
		
		Returns:
			list: x values for the pdf plot
			list: y values for the pdf plot
			
		"""

        mu = self.mean
        sigma = self.std

        min_range = min (self.data)
        max_range = max (self.data)

        interval = 1.0 * (max_range - min_range) / n_spaces

        x = []
        y = []

        for i in range(n_spaces):
            tmp = min_range + interval * i
            x.append(tmp)
            y.append(self.pdf(tmp))


        fig, axes = plt.subplots(2, sharex=True)
        fig.subplots_adjust(hspace=.5)
        axes[0].hist(self.data, density = True)
        axes[0].set_title('Normed Histogram of Data')
        axes[0].set_ylabel('Density')

        axes[1].plot(x, y)
        axes[1].set_title('Normed Distribution for sample mean and sample standard deviation')
        axes[1].set_ylabel('Density')

        plt.show()

        return x, y



from setuptools import setup

setup (name = 'probability_dstn', 
    version='0.2',
    description= 'Gaussian distributions and binomial distribution',
    packages= ['probability_dstn'],
    author= 'Okeowo Tunde',
    author_email ='okeowosamuel58.to@gmail.com',
    zip_safe = False)


     